#include "config.hpp"
#include <Arduino.h>
#include <Ticker.h>


/*===== Platform =====*/
/*[WIRELESS_SUPPORT]*/
#ifdef WIRELESS_SUPPORT
    #include "platform/esp_platform.hpp"
    #include <ArduinoOTA.h>
#endif


/*[NO_WIFI]*/
#if PLATFORM == NO_WIFI

#endif



/*===== Variable =====*/
//-300: off
double temp1_current = 0;
double temp1_target = HEAT_OFF;
double temp2_current = 0;
double temp2_target = HEAT_OFF;

String inputString = "";         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete



/*===== Function =====*/

double getTemperature(uint8_t thermistor_pin){
    // vin ---- Rs --(adc)-- Rtm ---- gnd 
    const double Vin = 3.3; //3.3 for ESP/STM32 series
    const double Rs = 10000; //strip resistor resistance
    const double Rtm_std = 100000; //thermistor resistance at 25C
    const uint32_t adc_resolution = 1023; //10bit ADC
    const uint32_t B = 3950;

    const double C1 = 1/298.15;

    // Vtm(Vadc) = Vin * Rtm/(Rs+Rtm)
    // Rtm = Vtm * Rs / (Vin - Vtm)

    // Rtm = Rtm_std * exp(B*(1/Tk - 1/298.15))
    // Rtm = Rtm_std * exp(B*(1/Tk - C1))

    // exp(B*(1/Tk - C1)) = Rtm / Rtm_std
    // B*(1/Tk - C1) = log[Rtm / Rtm_std]
    // 1/Tk = log[Rtm / Rtm_std] / B + C1
    // Tk = 1/( log[Rtm / Rtm_std] / B + C1 )

    // T = Tk -273.15;

    double Vtm = (analogRead(thermistor_pin) * Vin) / adc_resolution;
    double Rtm = Vtm * Rs / (Vin - Vtm);
    double T = 1 / ( log(Rtm / Rtm_std) / B + C1) - 273.15;

    return T;
}


void heat_drive(uint8_t heater, bool heat_sw){
    if(heater == 1){
        digitalWrite(HEATER1_PIN, heat_sw);
        digitalWrite(HEATER1_LED_PIN, !heat_sw); //low: turn on
    }else if(heater == 2){
        digitalWrite(HEATER2_PIN, heat_sw);
        digitalWrite(HEATER2_LED_PIN, !heat_sw); //low: turn on
    }
}


void heat_handler(){
    temp1_current = getTemperature(HEATER1_THERMISTOR_PIN);
    temp2_current = getTemperature(HEATER2_THERMISTOR_PIN);

    if(temp1_target > MAX_TEMP || temp1_target < 0 || temp1_current > temp1_target){
        heat_drive(1,0);
    }else{
        heat_drive(1,1);
    }

    if(temp2_target > MAX_TEMP || temp2_target < 0 || temp2_current > temp2_target){
        heat_drive(2,0);
    }else{
        heat_drive(2,1);
    }
}


#ifndef SERIAL_DEBUG

    void getCommand(){
        String commandString = "";

        if(inputString.length() > 0){
            commandString = inputString.substring(1,6);
        }

        if(commandString.equals("TARTP")){ //absolutely set temperature
            String abs_temp_str = inputString.substring(6,inputString.length());
            temp1_target = abs_temp_str.toInt();
        }

        if(commandString.equals("DRCTP")){ //relatively set temperature
            String rel_temp_str = inputString.substring(6,inputString.length());
            temp1_target += rel_temp_str.toInt();
        }

        inputString="";
    }


    void serialEvent(){
        while(Serial.available() && !stringComplete){
            // get the new byte:
            char inChar = (char)Serial.read();
            // add it to the inputString:
            inputString += inChar;
            // if the incoming character is a newline, set a flag
            // so the main loop can do something about it:
            if(inChar == '\n'){
            stringComplete = true;
            }
        }
    }


    void serial_report_handler(){
        Serial.println(temp1_target);  
        Serial.print(",");  
        Serial.print(temp1_current);  
        Serial.print(",");   
        Serial.print(temp2_target);  
        Serial.print(",");  
        Serial.print(temp2_current);  
        Serial.print(",");
    }

#endif



/*===== Object =====*/
#ifdef WIRELESS_SUPPORT
    Ticker heater;
    Ticker serial_reporter;
#else
    Ticker heater(heat_handler, 100);
    Ticker serial_reporter(serial_report_handler, 500);
#endif



/*===== Main =====*/

void setup() {
    Serial.begin(9600);

    /*[Pin Init]*/
    pinMode(HEAT_GUN_FAN_PIN, OUTPUT);
    pinMode(HEATER1_PIN, OUTPUT);
    pinMode(HEATER2_PIN, OUTPUT);
    pinMode(CONNECTION_LED_PIN, OUTPUT);
    pinMode(HEATER1_LED_PIN, OUTPUT);
    pinMode(HEATER2_LED_PIN, OUTPUT);

    #ifdef WIRELESS_SUPPORT
        wireless_setup();
    #endif

    /*[Ticker Init]*/
    #ifdef WIRELESS_SUPPORT
        heater.attach(0.1, heat_handler);
        #ifndef SERIAL_DEBUG
            serial_reporter.attach(0.5, serial_report_handler);
        #endif
    #else
        heater.start();
        #ifndef SERIAL_DEBUG
            serial_reporter.start();
        #endif
    #endif
    
}


void loop() {
    //temp1_current = rand()%100;
    //temp2_current = rand()%100;
    //delay(100);
    #ifdef WIRELESS_SUPPORT
        wireless_handler();
    #else
        heater.update();
        serial_reporter.update();
    #endif

    #ifndef SERIAL_DEBUG
        if(stringComplete){
            getCommand();
            stringComplete = false;
        }
    #endif
}